from typing import Dict, List

import click
import investor8_sdk
from pandas import DataFrame
from rich.console import Console
from rich.table import Table

from i8_terminal.commands.watchlist import watchlist
from i8_terminal.common.cli import pass_command
from i8_terminal.common.formatting import get_formatter
from i8_terminal.common.layout import df2Table, format_df
from i8_terminal.config import USER_SETTINGS

from i8_terminal.types.user_watchlists_param_type import UserWatchlistsParamType  # isort:skip


def get_watchlist_df(name: str) -> DataFrame:
    watchlist_stocks: List[Dict[str, str]] = []
    watchlist = investor8_sdk.UserApi().get_watchlist_by_name_user_id(name=name, user_id=USER_SETTINGS.get("user_id"))
    all_stock_info = investor8_sdk.StockInfoApi().get_all_stock_info(market_index="")
    all_latest_prices = investor8_sdk.PriceApi().get_all_latest_prices()
    all_latest_daily_metrics = investor8_sdk.MetricsApi().get_all_latest_daily_metrics()
    for ticker in watchlist.tickers:
        latest_price = all_latest_prices[ticker]
        stock_info = all_stock_info[ticker]
        daily_metrics = all_latest_daily_metrics[ticker]
        watchlist_stocks.append(
            {
                "ticker": ticker,
                "company": stock_info.name,
                "exchange": stock_info.exchange,
                "latest_price": latest_price.latest_price,
                "change": latest_price.change_perc,
                "market_cap": daily_metrics.marketcap,
            }
        )
    return DataFrame(watchlist_stocks)


def render_watchlist_table(df: DataFrame) -> Table:
    target = "console"
    formatters = {
        "latest_price": get_formatter("price", target),
        "change": get_formatter("perc", target),
        "market_cap": get_formatter("price", target),
    }
    col_names = {
        "ticker": "Ticker",
        "company": "Company",
        "exchange": "Exchange",
        "latest_price": "Price",
        "change": "Change",
        "market_cap": "Market Cap",
    }
    df_formatted = format_df(df, col_names, formatters)
    columns_justify = {
        "Price": "right",
        "Change": "right",
        "Market Cap": "right",
    }
    return df2Table(df_formatted, columns_justify=columns_justify)


@watchlist.command()
@click.option(
    "--name",
    "-n",
    type=UserWatchlistsParamType(),
    required=True,
    help="Name of the watchlist you want to see more details.",
)
@pass_command
def details(name: str) -> None:
    console = Console()
    with console.status("Fetching data...", spinner="material"):
        df = get_watchlist_df(name)
    table = render_watchlist_table(df)
    console.print(table)



from typing import Any, Dict, List

import click
import investor8_sdk
import pandas as pd
import plotly.express as px
import plotly.graph_objects as go
from pandas.core.frame import DataFrame
from rich.console import Console

from i8_terminal.app.layout import get_plot_default_layout
from i8_terminal.app.plot_server import serve_plot
from i8_terminal.commands.earnings import earnings
from i8_terminal.common.cli import get_click_command_path, pass_command
from i8_terminal.common.formatting import format_number
from i8_terminal.common.utils import PlotType
from i8_terminal.types.metric_param_type import MetricParamType
from i8_terminal.types.ticker_param_type import TickerParamType


def get_historical_earnings_df(tickers: List[str], size: int) -> DataFrame:
    hist_earnings = []
    for tk in tickers:
        hist_earnings.extend(investor8_sdk.EarningsApi().get_historical_earnings(tk, size=size))
    df = pd.DataFrame([h.to_dict() for h in hist_earnings])[
        [
            "ticker",
            "fyq",
            "eps_actual",
            "eps_beat_rate",
            "eps_surprise",
            "eps_ws",
            "revenue_actual",
            "revenue_beat_rate",
            "revenue_surprise",
            "revenue_ws",
        ]
    ]
    df = df.sort_values(by=["ticker", "fyq"], ascending=True).reset_index(drop=True)
    df.fyq = df.fyq.str[2:-2] + " " + df.fyq.str[-2:]
    df = df.rename(columns={"eps_actual": "Actual", "eps_ws": "Consensus"})
    return df


def create_fig(df: DataFrame, cmd_context: Dict[str, Any]) -> go.Figure:
    fig = px.bar(
        df,
        hover_data=[],
        x="fyq",
        y=["Actual", "Consensus"],
        barmode="group",
        title=cmd_context["plot_title"],
        labels={"value": "Metric Value", "fyq": "Period"},
    )
    fig.update_xaxes(
        tickvals=df.fyq.to_list(),
        ticktext=df.apply(lambda x: x.fyq + f"<br>Beat by ${format_number(x.eps_surprise, decimal=2)}", axis=1),
    )
    fig.update_traces(width=0.3, hovertemplate="%{y}%{_xother}")
    fig.update_layout(
        **get_plot_default_layout(),
        bargap=0.4,
        legend_title_text=None,
        xaxis_title=None,
        margin=dict(b=15, l=70, r=20),
    )

    return fig


@earnings.command()
@click.pass_context
@click.option("--metric", "-m", type=MetricParamType(), default="basiceps", help="Metric name.")
@click.option("--tickers", "-k", type=TickerParamType(), required=True, help="Comma-separated list of tickers.")
@pass_command
def plot(ctx: click.Context, metric: str, tickers: str) -> None:
    """Compare and plot earning metrics of given companies. TICKERS is a comma seperated list of tickers.

    Examples:

    `i8 earnings plot AMD,INTC,QCOM`
    """
    metric = metric.replace(" ", "").upper()
    tickers_list = tickers.replace(" ", "").upper().split(",")
    command_path = get_click_command_path(ctx)
    tickers_list = tickers.replace(" ", "").upper().split(",")
    plot_title = f"{', '.join(tickers_list)} - {metric} Surprise & Estimates by Quarter"
    cmd_context = {
        "command_path": command_path,
        "tickers": tickers_list,
        "plot_title": plot_title,
        "plot_type": PlotType.CHART.value,
    }

    console = Console()
    with console.status("Fetching data...", spinner="material") as status:
        df = get_historical_earnings_df(tickers_list, size=4)

        status.update("Generating plot...")
        fig = create_fig(df, cmd_context)

    serve_plot(fig, cmd_context)



from datetime import datetime
from typing import Any, Dict, List, Optional, cast

import arrow
import click
import investor8_sdk
import numpy as np
import plotly.express as px
import plotly.graph_objects as go
from click.types import DateTime
from pandas import DataFrame
from plotly.subplots import make_subplots
from rich.console import Console

from i8_terminal.app.layout import get_plot_default_layout
from i8_terminal.app.plot_server import serve_plot
from i8_terminal.commands.financials import financials
from i8_terminal.common.cli import get_click_command_path, pass_command
from i8_terminal.common.utils import PlotType
from i8_terminal.types.metric_param_type import MetricParamType
from i8_terminal.types.period_type_param_type import PeriodTypeParamType
from i8_terminal.types.ticker_param_type import TickerParamType

from i8_terminal.common.metrics import find_similar_fin_metric, get_metrics_display_names  # isort:skip
from i8_terminal.types.chart_param_type import ChartParamType, get_chart_param_types  # isort:skip


def get_historical_financials_df(
    tickers: List[str], metrics: List[str], period_type: str, from_date: Optional[str], to_date: Optional[str]
) -> DataFrame:
    if not to_date:
        to_date = arrow.now().datetime.strftime("%Y-%m-%d")
    if not from_date:
        from_date = arrow.now().shift(years=-8 if period_type == "FY" else -4).datetime.strftime("%Y-%m-%d")
    hist_financials = investor8_sdk.MetricsApi().get_historical_metrics(
        tickers=",".join(tickers),
        metrics=",".join(metrics),
        period_type=period_type,
        from_date=from_date,
        to_date=to_date,
    )
    df = DataFrame.from_dict(
        {(i, j): hist_financials[i][j] for i in hist_financials.keys() for j in sorted(hist_financials[i].keys())}
    ).sort_index()
    df.index.name = f"Historical {' and '.join(get_metrics_display_names(metrics))}"
    return df


def create_fig(df: DataFrame, cmd_context: Dict[str, Any], matched_metrics: List[str], chart_type: str) -> go.Figure:
    vertical_spacing = 0.02
    layout = dict(
        title=cmd_context["plot_title"],
        autosize=True,
        hovermode="closest",
        legend=dict(font=dict(size=11), orientation="v"),
        margin=dict(b=20, l=50, r=65),
    )
    rows_num = len(matched_metrics)

    if rows_num == 2:
        row_width = [0.5, 0.5]
    else:
        row_width = [1]

    fig = make_subplots(
        rows=rows_num, cols=1, shared_xaxes=True, vertical_spacing=vertical_spacing, row_width=row_width
    )

    for m in matched_metrics:
        for idx, tk in enumerate(df[m].columns):
            if chart_type == "bar":
                fig.add_trace(
                    go.Bar(
                        x=df[m].index,
                        y=df[m][tk],
                        name=tk,
                        marker=dict(color=px.colors.qualitative.Plotly[idx]),
                        legendgroup=f"group{idx}",
                        showlegend=True if matched_metrics.index(m) == 0 else False,
                    ),
                    row=matched_metrics.index(m) + 1,
                    col=1,
                )
            else:
                fig.add_trace(
                    go.Scatter(
                        x=df[m].index,
                        y=df[m][tk],
                        name=tk,
                        marker=dict(color=px.colors.qualitative.Plotly[idx]),
                        legendgroup=f"group{idx}",
                        showlegend=True if matched_metrics.index(m) == 0 else False,
                    ),
                    row=matched_metrics.index(m) + 1,
                    col=1,
                )

    fig.update_traces(hovertemplate="%{y} %{x}")
    fig.update_xaxes(
        rangeslider_visible=False,
        spikemode="across",
        spikesnap="cursor",
    )

    fig.update_layout(
        **layout,
        **get_plot_default_layout(),
        legend_title_text=None,
        xaxis_title=None,
        yaxis_title=None,
    )

    # Add yaxis titles
    for idx, r in enumerate(np.cumsum(row_width)[::-1]):
        fig["layout"][f"yaxis{idx+1}"]["title"] = get_metrics_display_names(matched_metrics)[idx]

    fig.update_annotations(
        dict(
            font_size=10,
            font_color="#525252",
        )
    )
    fig.update_yaxes(
        title_font_size=10,
    )

    return fig


@financials.command()
@click.pass_context
@click.option("--tickers", "-k", type=TickerParamType(), required=True, help="Comma-separated list of tickers.")
@click.option(
    "--period_type",
    "-m",
    type=PeriodTypeParamType(),
    default="FY",
    help="Period by which you want to view the report. Possible values are `FY` for yearly, `Q` for quarterly, and `TTM` for TTM reports.",
)
@click.option("--metrics", "-m", type=MetricParamType(), default="basic_eps", help="Comma-separated list of metrics.")
@click.option("--from_date", "-f", type=DateTime(), help="Histotical financials from date.")
@click.option("--to_date", "-t", type=DateTime(), help="Histotical financials to date.")
@click.option(
    "--chart_type",
    "-c",
    type=ChartParamType([("bar", "Bar chart"), ("line", "Line chart")]),
    default="bar",
    help="Chart can be bar or line chart.",
)
@pass_command
def plot(
    ctx: click.Context,
    tickers: str,
    period_type: str,
    metrics: str,
    chart_type: str,
    from_date: Optional[datetime],
    to_date: Optional[datetime],
) -> None:
    """Compare and plot financials metrics of given companies. TICKERS is a comma seperated list of tickers.

    Examples:

    `i8 financials plot AMD,INTC,QCOM`
    """
    metrics_list = metrics.replace(" ", "").split(",")
    matched_metrics = [find_similar_fin_metric(metric.replace("_", "")) for metric in metrics_list]
    if not matched_metrics:
        click.echo(
            f"`{metrics}` is not valid metrics name. See the list of valid financial metrics with the following command:\n`i8 metrics`"
        )
        return
    if len(matched_metrics) > 2:
        click.echo("You can enter up to 2 metrics.")
        return
    if not chart_type in [t[0] for t in get_chart_param_types()]:
        click.echo(f"`{chart_type}` is not valid chart type.")
        return
    command_path_parsed_options_dict = {"--metrics": ",".join(matched_metrics)}  # type: ignore
    if from_date:
        command_path_parsed_options_dict["--from_date"] = from_date.strftime("%Y-%m-%d")
    if to_date:
        command_path_parsed_options_dict["--to_date"] = to_date.strftime("%Y-%m-%d")
    command_path = get_click_command_path(ctx, command_path_parsed_options_dict)
    tickers_list = tickers.replace(" ", "").upper().split(",")
    if len(tickers_list) > 5:
        click.echo("You can enter up to 5 tickers.")
        return
    cmd_context = {
        "command_path": command_path,
        "tickers": tickers_list,
        "plot_type": PlotType.CHART.value,
    }

    console = Console()
    with console.status("Fetching data...", spinner="material") as status:
        df = get_historical_financials_df(
            tickers_list, matched_metrics, period_type, cast(str, from_date), cast(str, to_date)  # type: ignore
        )
        cmd_context["plot_title"] = df.index.name
        status.update("Generating plot...")
        fig = create_fig(df, cmd_context, matched_metrics, chart_type)  # type: ignore

    serve_plot(fig, cmd_context)



from typing import Any, Dict, List

import click
import investor8_sdk
from pandas import DataFrame
from rich.console import Console

from i8_terminal.commands.news import news
from i8_terminal.common.cli import pass_command
from i8_terminal.common.formatting import format_number, get_formatter
from i8_terminal.common.layout import df2Table, format_df
from i8_terminal.common.utils import validate_ticker
from i8_terminal.types.ticker_param_type import TickerParamType


def get_news_df(identifier: str, page_size: int) -> DataFrame:
    if identifier:
        news = investor8_sdk.NewsApi().get_ticker_news(identifier)
    else:
        news = investor8_sdk.NewsApi().get_latest_news(page_size=page_size)
    df = DataFrame([d.to_dict() for d in news])
    df["news_source"] = df["news_source"].apply(map_news_source)
    df["stock_prices"] = df["stock_prices"].apply(lambda x: format_stock_prices(x))
    return df


def format_stock_prices(prices: List[Dict[str, Any]]) -> str:
    formatted_stock_prices = []
    for p in prices:
        formatted_stock_prices.append(
            f'{p["ticker"]} {format_number(p["latest_price"])} ({format_number(p["change_perc"], unit="percentage")})'
        )
    return "\n".join(formatted_stock_prices)


def format_news_df(df: DataFrame, target: str) -> DataFrame:
    formatters = {
        "publication_timestamp": get_formatter("date", target),
    }
    col_names = {
        "news_source": "Source",
        "publication_timestamp": "Date",
        "title": "Title",
        "stock_prices": "Tickers",
    }
    return format_df(df, col_names, formatters)


def map_news_source(news_source: int) -> str:
    if news_source == 0:
        return "InvestorEight"
    elif news_source == 1:
        return "Yahoo!Finance"
    elif news_source == 2:
        return "Highlights"
    else:
        return "-"


@news.command()
@click.option(
    "--ticker", "-k", type=TickerParamType(), required=True, callback=validate_ticker, help="Ticker or company name."
)
@pass_command
def list(ticker: str) -> None:
    """Lists the latest market news. If ticker is provided, the news are filtered based on the give ticker."""
    console = Console()
    with console.status("Fetching data...", spinner="material"):
        df = get_news_df(ticker, page_size=10)
    df_formatted = format_news_df(df, "console")
    table = df2Table(df_formatted)
    console.print(table)



from datetime import datetime
from time import sleep
from typing import Optional

import arrow
import click
import investor8_sdk
import pandas as pd
from pandas.core.frame import DataFrame
from rich.console import Console
from rich.layout import Layout
from rich.live import Live
from rich.panel import Panel
from rich.table import Table

from i8_terminal.commands.market import market
from i8_terminal.common.cli import pass_command
from i8_terminal.common.formatting import get_formatter
from i8_terminal.common.layout import df2Table, format_df


def make_layout() -> Layout:
    """Define the layout."""
    layout = Layout(name="root")
    layout["root"].split(Layout(name="main"))
    layout["main"].split_row(
        Layout(name="side_left"),
        Layout(name="side_right"),
    )
    layout["side_left"].split(
        Layout(name="title", minimum_size=8, ratio=1),
        Layout(name="major_indices", minimum_size=9, ratio=1),
        Layout(name="today_winners", minimum_size=11),
    )
    layout["side_right"].split(
        Layout(name="sectors", minimum_size=17, ratio=2), Layout(name="today_losers", minimum_size=11)
    )

    return layout


def format_price_df(df: DataFrame, target: str, is_index: bool = False) -> DataFrame:
    formatters = {
        "latest_price": get_formatter("number", target),
        "change_perc": get_formatter("perc", target),
    }
    col_names = {
        "ticker": "Ticker",
        "latest_price": "Level" if is_index else "Price",
        "change_perc": "Change (%)",
    }
    if "name" in df.columns:
        df["name"] = df["name"].apply(lambda x: x if len(x) < 20 else x[:17] + "...")
        col_names["name"] = "Index" if is_index else "Company Name"
    if "price_time" in df.columns:
        col_names["price_time"] = "price_time"
    return format_df(df, col_names, formatters)


def format_sector_df(df: DataFrame, target: str) -> DataFrame:
    formatters = {
        "_return": get_formatter("perc", target),
    }
    col_names = {
        "sector": "Sector",
        "_return": "Change (%)",
    }
    return format_df(df, col_names, formatters)


def get_major_indices_df() -> Optional[DataFrame]:
    indices = {}
    try:
        indices = investor8_sdk.PriceApi().get_latest_market_indices()
    except Exception as e:
        click.echo(e.body)  # type: ignore
        return None

    df = pd.DataFrame([{"name": k, **v.to_dict()} for k, v in indices.items()])[
        ["ticker", "name", "latest_price", "change_perc"]
    ]
    return format_price_df(df, "console", is_index=True)


def get_today_top_stocks_df(top_stocks_type: str, count: int = 5) -> Optional[DataFrame]:
    stocks = []
    try:
        stocks = investor8_sdk.ScreenerApi().get_top_stocks(top_stocks_type, count=count, index="$SPX")
    except Exception as e:
        click.echo(e.body)  # type: ignore
        return None
    df = pd.DataFrame([d.to_dict() for d in stocks])[["ticker", "name", "latest_price", "change_perc"]]

    return format_price_df(df, "console")


def get_sector_returns_df() -> Optional[DataFrame]:
    sectors = None
    try:
        sectors = investor8_sdk.ScreenerApi().get_all_sectors_returns(period=1)
    except Exception as e:
        click.echo(e.body)  # type: ignore
        return None

    df = pd.DataFrame([d.to_dict() for d in sectors["1D"]])[["sector", "_return"]]

    return format_sector_df(df, "console")


def get_today_intraday_prices_df(tickers: str, indices: str, size: int = 10) -> Optional[DataFrame]:
    all_prices = {}  # type: ignore
    for tk_list in [tickers, indices]:
        try:
            prices = investor8_sdk.PriceApi().get_today_intraday_prices(tickers=tk_list, size=size)
        except Exception as e:
            click.echo(e.body)  # type: ignore
            return None
        all_prices = {**all_prices, **prices}
    prices_list = []
    for v in list(all_prices.values()):
        for p in v:
            prices_list.append(p.to_dict())
    df = pd.DataFrame(prices_list)[["ticker", "latest_price", "change_perc", "price_time"]]

    return format_price_df(df, "console")


def get_title_table(date: Optional[datetime] = None) -> Table:
    grid = Table.grid(expand=True)
    grid.add_column(justify="left", ratio=2)
    grid.add_row("[b]Investoreight[/b] Market Summary")
    dt = (
        date.strftime("%b %d %Y, %I:%M:%S %p ET").replace(":", "[blink]:[/]")
        if date
        else arrow.now("US/Eastern").datetime.strftime("%b %d %Y, %I:%M:%S %p ET").replace(":", "[blink]:[/]")
    )
    grid.add_row(
        ":stopwatch:  [magenta]" + dt,
    )
    return grid


def get_market_summary_panels(name: str, df: DataFrame) -> Panel:
    return {
        "major_ind": Panel(
            df2Table(df),
            border_style="blue",
            title="Major U.S. Indices",
        ),
        "winners": Panel(
            df2Table(df),
            border_style="green",
            title="S&P500 Winners",
        ),
        "losers": Panel(
            df2Table(df),
            border_style="red",
            title="S&P500 Losers",
        ),
    }[name]


@market.command()
@click.option("--live", is_flag=True, default=False, help="Print live results of market summary on terminal.")
@pass_command
def summary(live: bool) -> None:
    console = Console()
    price_size = 10
    with console.status("Fetching data...", spinner="material") as status:
        sectors_df = get_sector_returns_df()
        major_indices_df = get_major_indices_df()
        today_winners_df = get_today_top_stocks_df("winners")
        today_losers_df = get_today_top_stocks_df("losers")
        if sectors_df is None or major_indices_df is None or today_winners_df is None or today_losers_df is None:
            return
        tickers = ",".join(set(today_winners_df.Ticker.to_list() + today_losers_df.Ticker.to_list()))
        indices = ",".join(set(major_indices_df.Ticker.to_list()))
        layout = make_layout()
        # Initialize layout
        dt_now = arrow.now("US/Eastern")
        layout["title"].update(Panel(get_title_table(dt_now.datetime), border_style="blue", padding=(2, 2)))
        layout["sectors"].update(Panel(df2Table(sectors_df), border_style="blue", title="U.S. Sectors"))
        intraday_prices = None
        if live:
            intraday_prices = get_today_intraday_prices_df(tickers, indices, price_size)
            if intraday_prices is not None:
                stock_cols = ["Ticker", "Price", "Change (%)", "Company Name"]
                index_cols = ["Index", "Level", "Change (%)"]
                renamed_cols = {"Price_y": "Price", "Change (%)_y": "Change (%)"}
                cols = ["Ticker", "Company Name", "Price_y", "Change (%)_y", "price_time"]
                today_winners_df = today_winners_df.merge(intraday_prices, on="Ticker")[cols]
                today_winners_df.rename(columns=renamed_cols, inplace=True)
                today_losers_df = today_losers_df.merge(intraday_prices, on="Ticker")[cols]
                today_losers_df.rename(columns=renamed_cols, inplace=True)
                major_indices_df = major_indices_df.merge(intraday_prices, on="Ticker")[
                    ["Ticker", "Index", "Price", "Change (%)_y", "price_time"]
                ]
                major_indices_df.rename(columns={"Price": "Level", "Change (%)_y": "Change (%)"}, inplace=True)
                # Initialize layout
                layout["major_indices"].update(
                    get_market_summary_panels(
                        "major_ind", major_indices_df.groupby("Index").nth(0).reset_index()[index_cols]
                    )
                )
                layout["today_winners"].update(
                    get_market_summary_panels(
                        "winners", today_winners_df.groupby("Ticker").nth(0).reset_index()[stock_cols]
                    )
                )
                layout["today_losers"].update(
                    get_market_summary_panels(
                        "losers",
                        today_losers_df.groupby("Ticker").nth(0).reset_index()[stock_cols],
                    )
                )
                status.stop()
                with Live(layout, refresh_per_second=4):
                    for i in range(2, price_size * 2):
                        dt_now = dt_now.shift(seconds=+1)
                        layout["title"].update(
                            Panel(get_title_table(dt_now.datetime), border_style="blue", padding=(2, 2))
                        )
                        if i % 2 == 0:  # Update prices every two secends
                            idx = int(i / 2)
                            layout["major_indices"].update(
                                get_market_summary_panels(
                                    "major_ind", major_indices_df.groupby("Index").nth(idx).reset_index()[index_cols]
                                )
                            )
                            layout["today_winners"].update(
                                get_market_summary_panels(
                                    "winners", today_winners_df.groupby("Ticker").nth(idx).reset_index()[stock_cols]
                                )
                            )
                            layout["today_losers"].update(
                                get_market_summary_panels(
                                    "losers", today_losers_df.groupby("Ticker").nth(idx).reset_index()[stock_cols]
                                )
                            )
                        sleep(1)

        elif not live or (live and intraday_prices is None):
            status.stop()
            layout["major_indices"].update(
                Panel(df2Table(major_indices_df), border_style="blue", title="Major U.S. Indices")
            )
            layout["today_winners"].update(
                Panel(df2Table(today_winners_df), border_style="green", title="S&P500 Winners")
            )
            layout["today_losers"].update(Panel(df2Table(today_losers_df), border_style="red", title="S&P500 Losers"))
            console.print(layout)



from typing import Any, Dict

from pandas import DataFrame
from rich.table import Table

from i8_terminal.config import get_table_style


def format_df(df: DataFrame, cols_map: Dict[str, str], cols_formatters: Dict[str, Any]) -> DataFrame:
    for c, f in cols_formatters.items():
        df[c] = df[c].map(f)
    return df[cols_map.keys()].rename(columns=cols_map)


def df2Table(df: DataFrame, style_profile: str = "default", columns_justify: Dict[str, Any] = {}) -> Table:
    style = get_table_style(style_profile)
    table = Table(**style)
    for c in df.columns:
        table.add_column(c, justify=columns_justify.get(c, "left"))
    for _, r in df.iterrows():
        row = [r[c] for c in df.columns]
        table.add_row(*row)
    return table
